@@ -8,7 +8,7 @@
 #define MIN_VERSION_mtl(x,y,z) 1
 #endif
 
-#ifndef HASKELL98
+#ifdef MTL
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE UndecidableInstances #-}
@@ -70,7 +70,7 @@ import Control.Monad.Trans.Class
 import Control.Monad.Zip (MonadZip(mzipWith))
 #endif
 
-#ifndef HASKELL98
+#ifdef MTL
 import Control.Monad.Writer.Class
 import Control.Monad.State.Class
 import Control.Monad.Reader.Class
@@ -97,11 +97,13 @@ type Except e = ExceptT e Identity
 -- (The inverse of 'runExcept').
 except :: Either e a -> Except e a
 except m = ExceptT (Identity m)
+{-# INLINE except #-}
 
 -- | Extractor for computations in the exception monad.
 -- (The inverse of 'except').
 runExcept :: Except e a -> Either e a
 runExcept (ExceptT m) = runIdentity m
+{-# INLINE runExcept #-}
 
 -- | Map the unwrapped computation using the given function.
 --
@@ -110,11 +112,13 @@ mapExcept :: (Either e a -> Either e' b)
         -> Except e a
         -> Except e' b
 mapExcept f = mapExceptT (Identity . f . runIdentity)
+{-# INLINE mapExcept #-}
 
 -- | Transform any exceptions thrown by the computation using the given
 -- function (a specialization of 'withExceptT').
 withExcept :: (e -> e') -> Except e a -> Except e' a
 withExcept = withExceptT
+{-# INLINE withExcept #-}
 
 -- | A monad transformer that adds exceptions to other monads.
 --
@@ -131,10 +135,12 @@ newtype ExceptT e m a = ExceptT { runExceptT :: m (Either e a) }
 
 instance (Eq e, Eq1 m) => Eq1 (ExceptT e m) where
     liftEq eq (ExceptT x) (ExceptT y) = liftEq (liftEq eq) x y
+    {-# INLINE liftEq #-}
 
 instance (Ord e, Ord1 m) => Ord1 (ExceptT e m) where
     liftCompare comp (ExceptT x) (ExceptT y) =
         liftCompare (liftCompare comp) x y
+    {-# INLINE liftCompare #-}
 
 instance (Read e, Read1 m) => Read1 (ExceptT e m) where
     liftReadsPrec rp rl = readsData $
@@ -164,24 +170,30 @@ mapExceptT :: (m (Either e a) -> n (Either e' b))
         -> ExceptT e m a
         -> ExceptT e' n b
 mapExceptT f m = ExceptT $ f (runExceptT m)
+{-# INLINE mapExceptT #-}
 
 -- | Transform any exceptions thrown by the computation using the
 -- given function.
 withExceptT :: (Functor m) => (e -> e') -> ExceptT e m a -> ExceptT e' m a
 withExceptT f = mapExceptT $ fmap $ either (Left . f) Right
+{-# INLINE withExceptT #-}
 
 instance (Functor m) => Functor (ExceptT e m) where
     fmap f = ExceptT . fmap (fmap f) . runExceptT
+    {-# INLINE fmap #-}
 
 instance (Foldable f) => Foldable (ExceptT e f) where
     foldMap f (ExceptT a) = foldMap (either (const mempty) f) a
+    {-# INLINE foldMap #-}
 
 instance (Traversable f) => Traversable (ExceptT e f) where
     traverse f (ExceptT a) =
         ExceptT <$> traverse (either (pure . Left) (fmap Right . f)) a
+    {-# INLINE traverse #-}
 
 instance (Functor m, Monad m) => Applicative (ExceptT e m) where
     pure a = ExceptT $ return (Right a)
+    {-# INLINE pure #-}
     ExceptT f <*> ExceptT v = ExceptT $ do
         mf <- f
         case mf of
@@ -191,45 +203,59 @@ instance (Functor m, Monad m) => Applicative (ExceptT e m) where
                 case mv of
                     Left e -> return (Left e)
                     Right x -> return (Right (k x))
+    {-# INLINEABLE (<*>) #-}
+    m *> k = m >>= \_ -> k
+    {-# INLINE (*>) #-}
 
 instance (Functor m, Monad m, Monoid e) => Alternative (ExceptT e m) where
     empty = ExceptT $ return (Left mempty)
+    {-# INLINE empty #-}
     ExceptT mx <|> ExceptT my = ExceptT $ do
         ex <- mx
         case ex of
             Left e -> liftM (either (Left . mappend e) Right) my
             Right x -> return (Right x)
+    {-# INLINEABLE (<|>) #-}
 
 instance (Monad m) => Monad (ExceptT e m) where
     return a = ExceptT $ return (Right a)
+    {-# INLINE return #-}
     m >>= k = ExceptT $ do
         a <- runExceptT m
         case a of
             Left e -> return (Left e)
             Right x -> runExceptT (k x)
+    {-# INLINE (>>=) #-}
     fail = ExceptT . fail
+    {-# INLINE fail #-}
 
 instance (Monad m, Monoid e) => MonadPlus (ExceptT e m) where
     mzero = ExceptT $ return (Left mempty)
+    {-# INLINE mzero #-}
     ExceptT m `mplus` ExceptT n = ExceptT $ do
         a <- m
         case a of
             Left e -> liftM (either (Left . mappend e) Right) n
             Right x -> return (Right x)
+    {-# INLINEABLE mplus #-}
 
 instance (MonadFix m) => MonadFix (ExceptT e m) where
     mfix f = ExceptT (mfix (runExceptT . f . either (const bomb) id))
       where bomb = error "mfix (ExceptT): inner computation returned Left value"
+    {-# INLINE mfix #-}
 
 instance MonadTrans (ExceptT e) where
     lift = ExceptT . liftM Right
+    {-# INLINE lift #-}
 
 instance (MonadIO m) => MonadIO (ExceptT e m) where
     liftIO = lift . liftIO
+    {-# INLINE liftIO #-}
 
 #if MIN_VERSION_base(4,4,0)
 instance (MonadZip m) => MonadZip (ExceptT e m) where
     mzipWith f (ExceptT a) (ExceptT b) = ExceptT $ mzipWith (liftA2 f) a b
+    {-# INLINE mzipWith #-}
 #endif
 
 -- | Signal an exception value @e@.
@@ -239,6 +265,7 @@ instance (MonadZip m) => MonadZip (ExceptT e m) where
 -- * @'throwE' e >>= m = 'throwE' e@
 throwE :: (Monad m) => e -> ExceptT e m a
 throwE = ExceptT . return . Left
+{-# INLINE throwE #-}
 
 -- | Handle an exception.
 --
@@ -255,18 +282,21 @@ m `catchE` h = ExceptT $ do
     case a of
         Left  l -> runExceptT (h l)
         Right r -> return (Right r)
+{-# INLINE catchE #-}
 
 -- | Lift a @callCC@ operation to the new monad.
 liftCallCC :: CallCC m (Either e a) (Either e b) -> CallCC (ExceptT e m) a b
 liftCallCC callCC f = ExceptT $
     callCC $ \ c ->
     runExceptT (f (\ a -> ExceptT $ c (Right a)))
+{-# INLINE liftCallCC #-}
 
 -- | Lift a @listen@ operation to the new monad.
 liftListen :: (Monad m) => Listen w m (Either e a) -> Listen w (ExceptT e m) a
 liftListen listen = mapExceptT $ \ m -> do
     (a, w) <- listen m
     return $! fmap (\ r -> (r, w)) a
+{-# INLINE liftListen #-}
 
 -- | Lift a @pass@ operation to the new monad.
 liftPass :: (Monad m) => Pass w m (Either e a) -> Pass w (ExceptT e m) a
@@ -275,10 +305,11 @@ liftPass pass = mapExceptT $ \ m -> pass $ do
     return $! case a of
         Left l -> (Left l, id)
         Right (r, f) -> (Right r, f)
+{-# INLINE liftPass #-}
 
 -- incurring the mtl dependency for these avoids packages that need them introducing orphans.
 
-#ifndef HASKELL98
+#ifdef MTL
 
 instance Monad m => MonadError e (ExceptT e m) where
     throwError = throwE
